@@ -13,11 +13,15 @@
 #' object. Defaults to \code{NULL}, as \code{UniLambda} is only required if you wish to
 #' compute \code{\link{ARPB}}.
 #' @param standardized lets the function know whether to look for standardized or
-#' unstandardized results from \pkg{lavaan}. If \code{Lambda} is not a \pkg{lavaan} object,
-#' then \code{standardized} will be ignored.
-#' @param Phi is the correlation matrix of factors. User should generally ignore this
-#' parameter. \code{bifactorIndices} will try to determine it from Lambda or will assume
-#' it is the identity matrix.
+#' unstandardized results from \pkg{lavaan} and defaults to \code{TRUE}. If \code{Lambda} is not a
+#'  \pkg{lavaan} object, then \code{standardized} will be ignored.
+#' @param Phi is the correlation matrix of factors and defaults to \code{NULL}. User should generally ignore this
+#' parameter. If not provided, \code{bifactorIndices} will try to determine \code{Phi} from \code{Lambda} when \code{Lambda}
+#' is a fitted lavaan model or will assume it is the identity matrix otherwise.
+#' @param Thresh is a list of vectors of item thresholds, used only when items are categorical.\code{bifactorIndices}
+#'  will try to determine \code{Thresh} from \code{Lambda} when \code{Lambda}
+#' is a fitted lavaan model and the indicators are categorical.
+#' \code{Thresh} defaults to null, which indicates items are continuous.
 #'
 #' @return A list of bifactor indices, including three different ECV indices, IECV, PUC,
 #' Omega, OmegaH, Factor Determinacy (FD), Construct Replicability (H) and ARPB.
@@ -38,11 +42,23 @@
 #' indices as well as details about their computation can be found in the man page for the
 #' individual indices.
 #'
+#' Formulas for all indices can be found in Rodriguez et al. (2016). When indicators are categorical,
+#' the methodology of Green and Yang (2009) is used for computing Omega and OmegaH.
+#'
 #' @references
+#' Green, S. B., & Yang, Y. (2009). Reliability of summed item scores using
+#' structural equation modeling: An alternative to coefficient alpha.
+#' \emph{Psychometrika, 74}(1), 155-167 \doi{10.1007/s11336-008-9099-3}.
+#'
 #' Kamata, A., & Bauer, D. J. (2008). A note on the relation between factor analytic and item
 #' response theory models. \emph{Structural Equation Modeling: A Multidisciplinary Journal, 15}
 #' (1), 136-153.
 #'
+#' #' Rodriguez, A., Reise, S. P., & Haviland, M. G. (2016). Evaluating bifactor models:
+#' calculating and interpreting statistical indices. \emph{Psychological Methods, 21}(2),
+#' 137 \doi{10.1037/met0000045}.
+#'
+
 #' @export
 #'
 #' @seealso
@@ -57,6 +73,8 @@
 #'          \code{\link{PUC}},
 #'          \code{\link{Omega_S}},
 #'          \code{\link{Omega_H}},
+#'          \code{\link{cat_Omega_S}},
+#'          \code{\link{cat_Omega_H}},
 #'          \code{\link{H}},
 #'          \code{\link{FD}},
 #'          \code{\link{ARPB}}
@@ -67,6 +85,7 @@
 #' # (using mirt object is similar). Use of the unidimensional
 #' # model is optional; it is only used to compute ARPB.
 #'
+#'\donttest{
 #'SRS_UnidimensionalModel <-
 #'   "SRS =~ SRS_1  + SRS_2  + SRS_3  + SRS_4  + SRS_5  +
 #'           SRS_6  + SRS_7  + SRS_8  + SRS_9  + SRS_10 +
@@ -95,7 +114,7 @@
 #'                             orthogonal = TRUE)
 #'
 #' bifactorIndices(SRS_bifactor, UniLambda = SRS_Unidimensional)
-#'
+#'}
 #'
 #'
 #' # Computing bifactor indices from standardized factor loading matrices
@@ -156,47 +175,77 @@
 #' bifactorIndices(MTMM_fit)
 #'
 
-bifactorIndices <- function(Lambda, Theta = NULL, UniLambda = NULL, standardized = TRUE, Phi = NULL) {
+bifactorIndices <- function(Lambda, Theta = NULL, UniLambda = NULL, standardized = TRUE, Phi = NULL, Thresh = NULL) {
+  ## If categorical and lavaan, then force standardized = TRUE and message about it
+  if (!standardized & "lavaan" %in% class(Lambda) ) {
+    if (length(lavaan::lavInspect(Lambda, "ordered")) > 0) {
+      standardized <- TRUE
+      message("Only bifactor indices based on standardized coefficients make sense for categorical indicators.")
+    }
+  }
+
+
   ## if fitted mirt object, then throw warning about Omegas probably being meaningless
   if ("SingleGroupClass" %in% class(Lambda)) {
-    warning("Interpreting omega indices for IRT models is not recommended at this time")
+    message("Interpreting omega indices for IRT models is not recommended at this time")
   }
 
   ## Make Lambda, Theta, Phi, and UniLambda matrices. Do Theta and Phi first because
   ## they need the Lambda object, not the Lambda matrix
-  if (is.null(Theta)) {Theta = getTheta(Lambda, standardized = standardized)}
+  if (is.null(Theta)) {Theta <- getTheta(Lambda, standardized = standardized)}
 
   # Can do Phi for SingleGroupClass and for lavaan
   if (is.null(Phi)) {
     if ("SingleGroupClass" %in% class(Lambda)) {
       Phi <- mirt::summary(Lambda)$fcor
     } else if ("lavaan" %in% class(Lambda)) {
-      Phi <- lavaan::lavInspect(Lambda, "std")$psi
+      Phi <- lavaan::lavInspect(Lambda, "std.lv")$psi
       # I hate that dumb symmetric matrix print method
       class(Phi) <- "matrix"
+    } else {
+      message("Latent variable covariance matrix is assumed to be the identity. This influences
+            FD and, in the case of categorical models, the omega indices.")
+      Phi <- diag(nrow = ncol(Lambda))  ## we can only get here if Lambda was provided as a matrix
+    }
+  }
+
+  # Can do Thresh for lavaan. Maybe we should do it for mirt as well.
+  if (is.null(Thresh) & ("lavaan" %in% class(Lambda))) {
+    # Check to see if items are ordered; if so, rip out the thresholds
+    if (length(lavaan::lavInspect(Lambda, "ordered")) > 0) {
+      thresh_long <- lavaan::lavInspect(Lambda, "std")$tau  ## if categorical, then standardized.
+      rownames(thresh_long) <- sapply(strsplit(rownames(thresh_long), "[|]"), "[[", 1)
+      items <- unique(rownames(thresh_long))
+      Thresh <- lapply(items, function (i) {
+        thresh_long[rownames(thresh_long) == i]
+      })
     }
   }
 
 
   Lambda <- getLambda(Lambda, standardized = standardized)
 
-  # If Phi is still NULL, let's make it the identity matrix and throw a warning
-  if (is.null(Phi)) {
-    Phi <- diag(nrow = ncol(Lambda))
-    warning("Factor intercorrelation matrix assumed to be the identity matrix for computation of FD.")
-  }
 
   if (!is.null(UniLambda)) {UniLambda <- getLambda(UniLambda, standardized = standardized)}
 
   ## Build up the lists of indices. FactorLevelIndices first
+
   FactorLevelIndices = list(ECV_SS  = ECV_SS(Lambda),
                             ECV_SG  = ECV_SG(Lambda),
-                            ECV_GS  = ECV_GS(Lambda),
-                            Omega   = Omega_S(Lambda, Theta),
-                            OmegaH  = Omega_H(Lambda, Theta))
+                            ECV_GS  = ECV_GS(Lambda))
+  if (is.null(Thresh)) {
+    FactorLevelIndices[["Omega"]]  <- Omega_S(Lambda, Theta)
+    FactorLevelIndices[["OmegaH"]] <- Omega_H(Lambda, Theta)
+  } else {
+    FactorLevelIndices[["Omega"]]  <- cat_Omega_S(Lambda, Thresh)
+    FactorLevelIndices[["OmegaH"]] <- cat_Omega_H(Lambda, Thresh)
+  }
+
   if (standardized) {
     FactorLevelIndices[["H"]] <- H(Lambda)
     FactorLevelIndices[["FD"]] <- FD(Lambda, Phi)
+  } else {
+    message("H and FD are currently only available when standardized = TRUE")
   }
 
   ## Remove any NULL values and convert to dataframe
@@ -223,8 +272,13 @@ bifactorIndices <- function(Lambda, Theta = NULL, UniLambda = NULL, standardized
   } else {
     Gen <- getGen(Lambda)
     ECV <- ECV_SG(Lambda)[Gen]
-    Omega <- Omega_S(Lambda, Theta)[Gen]
-    OmegaH <- Omega_H(Lambda, Theta)[Gen]
+    if (is.null(Thresh)) {
+      Omega <- Omega_S(Lambda, Theta)[Gen]
+      OmegaH <- Omega_H(Lambda, Theta)[Gen]
+    } else {
+      Omega <- cat_Omega_S(Lambda, Thresh)[Gen]
+      OmegaH <- cat_Omega_H(Lambda, Thresh)[Gen]
+    }
   }
 
   ModelLevelIndices <- c(ECV = ECV, PUC = PUC(Lambda), Omega = Omega, OmegaH  = OmegaH, ARPB = ARPB_indices[[1]])
@@ -234,7 +288,7 @@ bifactorIndices <- function(Lambda, Theta = NULL, UniLambda = NULL, standardized
                       FactorLevelIndices = FactorLevelIndices,
                       ItemLevelIndices   = ItemLevelIndices
                       )
-  ## if any index type is emtirely missing, remove that index type entirely (e.g., no model or item level indices if not bifactor)
+  ## if any index type is entirely missing, remove that index type entirely (e.g., no model or item level indices if not bifactor)
   indicesList[which(!sapply(indicesList, is.null))]
 
 }
@@ -259,7 +313,7 @@ bifactorIndices <- function(Lambda, Theta = NULL, UniLambda = NULL, standardized
 #' @details To use this function, simply call it without any arguments and a dialog box
 #' will pop up for you to select a .out file of a confirmatory bifactor model.
 #'
-#' ARPB will only be compute if the factor loadings from a unidimensional model
+#' ARPB will only be computed if the factor loadings from a unidimensional model
 #' (as a vector or as the result of using \code{\link[MplusAutomation]{readModels}} on an
 #' \code{Mplus} .out file) are included. Note that if a correlated traits model is provided,
 #' the omega indices will simply be the regular omega values for those factors. Interpretations
@@ -288,43 +342,80 @@ bifactorIndicesMplus <- function(Lambda = file.choose(), UniLambda = NULL, stand
   if ("character" %in% class(UniLambda)) {UniLambda <- MplusAutomation::readModels(UniLambda)}
 
   if (!("mplus.model" %in% class(Lambda))) {Lambda <- MplusAutomation::readModels(Lambda)}
-  ## if categorical, then error if standardized = FALSE and manually compute Theta if standardized = TRUE
-  categorical <- !is.null(Lambda$input$variable$categorical)
 
-  # if unstandardized and any factor has a variance other than 1, throw an error
-  if (!standardized) {
-    params <- Lambda$parameters$unstandardized
-    facVar <- params[params$paramHeader == "Variances","est"]
-      if (!all(facVar == 1)) {
-        stop("Bifactor indices require latent factors have variance = 1. Respecify your model or use standardized = TRUE")
-      }
+  ## Check if categorical indicators.
+  categorical <- !is.null(Lambda$input$variable$categorical)
+  ## categorical -> standardized
+  if (!standardized & categorical) {
+    standardized <- TRUE
+    message("Only bifactor indices based on standardized coefficients make sense for categorical indicators.")
   }
-  # Let's grab the interfactor correlation matrix, if we have stdyx results
-  if (is.null(Lambda$parameters$stdyx.standardized)) {
-    Phi <- NULL
-  } else {
+
+  # Fetch Phi matrix. Why isn't there a getPhi function?
+  if (standardized) {
+    # need to have standardized parameters if standardized!
+    if (is.null(Lambda$parameters$stdyx.standardized)) {
+      stop("You must request STDYX output for computing bifactor indices based on standardized coefficients.")
+    }
     params <- Lambda$parameters$stdyx.standardized
-    facNames <- params[params$paramHeader == "Variances", "param"]
-    factorCorrs <- lapply(1:length(facNames), function (x) {
-      fac <- facNames[x]
-      c(rep(0, x-1), .5, params[params$paramHeader == paste0(fac, ".WITH"), "est"])
-    })
-    factorCorrs <- matrix(unlist(factorCorrs), byrow=TRUE, nrow=length(factorCorrs) )
-    Phi <- factorCorrs + t(factorCorrs)
+  } else {
+    params <- Lambda$parameters$unstandardized
   }
 
+  ## We need factor names to be in the same order as factor loading matrix
+  facNames <- params[grep(".BY", params$paramHeader), "paramHeader"]
+  facNames <- gsub(".BY", "", facNames, fixed = TRUE)
+  facNames <- unique(facNames)
+  facVar <- sapply(facNames, function (fac) {
+    params[params$paramHeader == "Variances" & params$param == fac,"est"]
+  })
+
+  # grab factor correlations, make them more easily parsed, then grab them
+  factorCorrs <- params[grep(".WITH", params$paramHeader), ]
+  factorCorrs$paramHeader <- gsub(".WITH", "", factorCorrs$paramHeader, fixed = TRUE)
+  Phi <- lapply(1:length(facNames), function (x) {
+    fac1 <- facNames[x]
+    sapply(1:length(facNames), function (y) {
+      fac2 <- facNames[y]
+      ## Factor variances are different
+      if (x == y) {
+        facVar[x]
+      } else {
+        ## Look for (x,y) and if that's not there look for (y,x)
+        if (length(factorCorrs[factorCorrs$paramHeader == fac1 & factorCorrs$param == fac2, "est"]) == 1) {
+          factorCorrs[factorCorrs$paramHeader == fac1 & factorCorrs$param == fac2, "est"]
+        } else {
+          factorCorrs[factorCorrs$paramHeader == fac2 & factorCorrs$param == fac1, "est"]
+        }
+      }
+    })
+  })
+  Phi <- matrix(unlist(Phi), byrow=TRUE, nrow=length(Phi) )
+
   if (categorical) {
-    if (standardized) {
-      Lambda <- getLambda(Lambda, standardized = standardized)
-      Theta <- getTheta(Lambda, standardized = standardized)
-    } else {
-      stop("Bifactor indices based on unstandardized coefficients with categorical variables is not available")
-    }
+    Lambda <- getLambda(Lambda, standardized = standardized)
+    Theta <- getTheta(Lambda, standardized = standardized)
+    ## now get thresholds
+    items <- rownames(Lambda)
+    thresh_long <- params[params$paramHeader == "Thresholds",]
+    thresh_long$itemName <- sapply(strsplit(thresh_long$param, "[$]"), "[[", 1)
+    Thresh <- lapply(items, function (i) {
+      thresh_long[thresh_long$itemName == i, "est"]
+    })
   } else {
     Theta <- getTheta(Lambda, standardized = standardized)
     Lambda <- getLambda(Lambda, standardized = standardized)
+    Thresh <- NULL
   }
 
-  bifactorIndices(Lambda, Theta, UniLambda, standardized, Phi)
+  # if diag(Phi) is not all ones, then we are in trouble. Let's deal with that
+  D <- diag(x = sqrt(diag(Phi)))
+  Phi <- solve(D) %*% Phi %*% t(solve(D))
+  Lambda <- Lambda %*% solve(D)
+  colnames(Lambda) <- facNames
+
+
+
+  bifactorIndices(Lambda, Theta, UniLambda, standardized, Phi, Thresh)
 }
 
